
#############################################################
#
#   Program: mixed_model.R
#   Project: 
#   PI: 
#   Statistican/Programmer: Meridith Blevins, MS
#   Purpose: 
#   Notes: 
#   Created: 6 October 2011
#   Revisions: 
#     
#
#############################################################
library(rms)

## ODDS OF SEEKING HEALTH SERVICES
## GLM - try a normal logistic regression (not accounting for correlation) - then use Huber-White sandwich estimator
getsick$type_sickness.factor[is.na(getsick$type_sickness.factor)] <- "other (specify)"
getsick$type_sickness.factor[getsick$type_sickness.factor=="injury"] <- "other (specify)"
getsick$type_sickness.factor <- droplevels(getsick$type_sickness.factor)
getsick$distance_nearest_hospital.factor <- droplevels(getsick$distance_nearest_hospital.factor)
getsick$children_at_home <- impute(getsick$children_at_home,fun=median)
getsick$distance_travel[getsick$distance_travel>2] <- 3
lrm1.dd1 <- datadist(getsick)
options(datadist='lrm1.dd1')

lrm1_seek <- lrm(seek_care ~ sm_pilot + rcs(children_at_home,3) + distance_nearest_hospital.factor + type_sickness.factor, data=getsick, x=T, y=T)
lrm1_seek <- lrm(seek_care ~ sm_pilot + rcs(children_at_home,3) + distance_travel.factor + type_sickness.factor, data=getsick, x=T, y=T)
lrm1_seek <- lrm(seek_care ~ sm_pilot + children_at_home + as.factor(distance_travel) + type_sickness.factor, data=getsick, x=T, y=T)
lrm1_seek <- robcov(lrm1_seek,getsick$survey_number)
anova(lrm1_seek)
summary(lrm1_seek,type_sickness.factor="malaria")


## ODDS OF GOING TO MALINDI DISTRICT HOSPITAL
## GLM - try a normal logistic regression (not accounting for correlation) - then use Huber-White sandwich estimator
seekcare <- getsick[!is.na(getsick$seek_care) & getsick$seek_care.factor=="Yes",]
seekcare$outcome <- ifelse(seekcare$where_care.factor=="Malindi District Hospital",1,0)
lrm1_where <- lrm(outcome ~ sm_pilot + rcs(children_at_home,3) + distance_nearest_hospital.factor + severity2, data=seekcare)
lrm1_where <- lrm(outcome ~ sm_pilot + children_at_home + distance_nearest_hospital.factor + severity2, data=seekcare)
lrm1_where <- lrm(outcome ~ sm_pilot + rcs(children_at_home,3) + as.factor(distance_travel) + severity2, data=seekcare)
anova(lrm1_where)
lrm1_where <- lrm(outcome ~ sm_pilot + children_at_home + as.factor(distance_travel) + severity2, data=seekcare, x=T, y=T)
lrm1_where <- robcov(lrm1_where,seekcare$survey_number)
lrm1.dd1 <- datadist(seekcare)
options(datadist='lrm1.dd1')
anova(lrm1_where)
lrm1_labels <- c("Pilot","Children at home (per 1 child)","Distance travel for health servies","\\\\hspace{0.2in}0-5 km (ref)","\\\\hspace{0.2in}6-10 km","\\\\hspace{0.2in}>10 km","Severity","\\\\hspace{0.2in}Mild/Moderate (ref)","\\\\hspace{0.2in}Severe/Life threatening")
lrm1_sum1 <- summary(lrm1_where,severity2=1,sm_pilot=0,distance_travel=1)[seq(2,10,2),c(4,6,7)]
lrm1_sum2 <- paste(format(round(lrm1_sum1[,1],2),nsmall=2)," (",
                     format(round(lrm1_sum1[,2],2),nsmall=2)," to ",
                     format(round(lrm1_sum1[,3],2),nsmall=2),")",sep="")
lrm1_sum3 <- c(lrm1_sum2[1:2],"","1",lrm1_sum2[4:5],"","1",lrm1_sum2[3])
(pval.g2 <- sapply(as.data.frame(anova(lrm1_where))$P,myformat4)) 
latex.g2 <-paste(lrm1_labels,lrm1_sum3,
               c(pval.g2[c(1,2,3)],rep("",3),pval.g2[4],rep("",2)),sep=" & ")
lrm1 <- paste(latex.g2,collapse=" \\\\\\\\ ")


## ODDS OF GOING TO MALINDI DISTRICT HOSPITAL
## GLM - try a normal logistic regression (not accounting for correlation) - then use Huber-White sandwich estimator
lrm2_where <- lrm(outcome ~ sm_pilot + rcs(children_at_home,3) + distance_nearest_hospital.factor + type_sickness.factor, data=seekcare)
lrm2_where <- lrm(outcome ~ sm_pilot + children_at_home + distance_nearest_hospital.factor + type_sickness.factor, data=seekcare)
lrm2_where <- lrm(outcome ~ sm_pilot + rcs(children_at_home,3) + as.factor(distance_travel) + type_sickness.factor, data=seekcare)
anova(lrm2_where)
lrm2_where <- lrm(outcome ~ sm_pilot + children_at_home + as.factor(distance_travel) + type_sickness.factor, data=seekcare, x=T, y=T)
lrm2_where <- robcov(lrm2_where,seekcare$survey_number)
lrm2.dd1 <- datadist(seekcare)
options(datadist='lrm2.dd1')
anova(lrm2_where)
lrm2_labels <- c("Pilot","Children at home (per 1 child)","Distance travel for health servies","\\\\hspace{0.2in}0-5 km (ref)","\\\\hspace{0.2in}6-10 km","\\\\hspace{0.2in}>10 km","Sickness","\\\\hspace{0.2in}Malaria (ref)","\\\\hspace{0.2in}Stomach","\\\\hspace{0.2in}Respiratory","\\\\hspace{0.2in}Fever","\\\\hspace{0.2in}Other")
lrm2_sum1 <- summary(lrm2_where,type_sickness.factor="malaria",sm_pilot=0,distance_travel=1)[seq(2,16,2),c(4,6,7)]
lrm2_sum2 <- paste(format(round(lrm2_sum1[,1],2),nsmall=2)," (",
                     format(round(lrm2_sum1[,2],2),nsmall=2)," to ",
                     format(round(lrm2_sum1[,3],2),nsmall=2),")",sep="")
lrm2_sum3 <- c(lrm2_sum2[1:2],"","1",lrm2_sum2[3:4],"","1",lrm2_sum2[5:8])
(pval.g2 <- sapply(as.data.frame(anova(lrm2_where))$P,myformat4)) 
latex.g2 <-paste(lrm2_labels,lrm2_sum3,
               c(pval.g2[c(1,2,3)],rep("",3),pval.g2[4],rep("",5)),sep=" & ")
lrm2 <- paste(latex.g2,collapse=" \\\\\\\\ ")

## ODDS OF HOSPITAL-BASED DELIVERY
## GLM - try a normal logistic regression (not accounting for correlation) - then use Huber-White sandwich estimator
getpreg$outcome <- ifelse(!is.na(getpreg$where_delivered),getpreg$where_delivered-1,NA)
getpreg$children_at_home <- impute(getpreg$children_at_home,fun=median)
getpreg$distance_travel[getpreg$distance_travel>2] <- 3
lrm3.dd1 <- datadist(getpreg)
options(datadist='lrm3.dd1')
lrm3_preg <- lrm(outcome ~ sm_pilot + maternal_education + children_at_home + distance_nearest_hospital.factor, data=getpreg)
lrm3_preg <- lrm(outcome ~ sm_pilot + rcs(maternal_education,3) + rcs(children_at_home,4) + distance_nearest_hospital.factor + complications.factor, data=getpreg,x=T,y=T)
lrm3_preg <- lrm(outcome ~ sm_pilot + rcs(maternal_education,3) + rcs(children_at_home,4) + as.factor(distance_travel) + complications.factor, data=getpreg,x=T,y=T)
lrm3_preg <- robcov(lrm3_preg, getpreg$survey_number)
anova(lrm3_preg)
summary(lrm3_preg)
lrm3_labels <- c("Pilot","Maternal Education","\\\\hspace{0.2in}2 years (ref)","\\\\hspace{0.2in}4 years","\\\\hspace{0.2in}8 years","Children at home","\\\\hspace{0.2in}2 children","\\\\hspace{0.2in}6 children (ref)","\\\\hspace{0.2in}10 children","Distance travel for health servies","\\\\hspace{0.2in}0-5 km (ref)","\\\\hspace{0.2in}6-10 km","\\\\hspace{0.2in}>10 km","Birth complications")
lrm3_sum1 <- rbind(summary(lrm3_preg,maternal_education=c(2,4),children_at_home=c(6,2),sm_pilot=0,distance_travel=1)[c(2,4),c(4,6,7)],
		   summary(lrm3_preg,maternal_education=c(2,6),children_at_home=c(6,2),sm_pilot=0,distance_travel=1)[c(4,6),c(4,6,7)],
		   summary(lrm3_preg,maternal_education=c(2,6),children_at_home=c(6,10),sm_pilot=0,distance_travel=1)[seq(6,12,2),c(4,6,7)])
lrm3_sum2 <- paste(format(round(lrm3_sum1[,1],2),nsmall=2)," (",
                     format(round(lrm3_sum1[,2],2),nsmall=2)," to ",
                     format(round(lrm3_sum1[,3],2),nsmall=2),")",sep="")
lrm3_sum3 <- c(lrm3_sum2[1],"","1",lrm3_sum2[2:3],"",lrm3_sum2[4],"1",lrm3_sum2[5],"","1",lrm3_sum2[6:8])
(pval.g3 <- sapply(as.data.frame(anova(lrm3_preg))$P,myformat4)) 
latex.g3 <-paste(lrm3_labels,lrm3_sum3,
               c(pval.g3[c(1,2)],rep("",3),pval.g3[4],rep("",3),pval.g3[6],rep("",3),pval.g3[7]),sep=" & ")
lrm3 <- paste(latex.g3,collapse=" \\\\\\\\ ")




