myformat1 <- function(x){format(round(x,1),nsmall=1)}
myformat2 <- function(x){format(round(x,2),nsmall=2)}
myformat3 <- function(x){
    y <- NULL
    if(x < 0.01){y <- "<0.01"}
    else{y <- format(round(x,2),nsmall=2)}
    return(y)
}
myformat4 <- function(x){
    y <- NULL
    if(x < 0.001){y <- "<0.001"}
    else{y <- format(round(x,3),nsmall=3)}
    return(y)
}
myper0 <- function(x){
    y <- NULL
    if(x <= 0.5 & x!=0){y <- "<1"}
    else{y <- format(round(x),nsmall=0)}
    return(y)
}
myper1 <- function(x){
    y <- NULL
    if(x < 0.05 & x!=0){y <- "<0.1"}
    else{y <- format(round(x,1),nsmall=1)}
    return(y)
}
medianIQR <- function(var,digits=1){
  paste(format(round(median(var,na.rm=TRUE),digits),nsmall=digits)," (",format(round(quantile(var,0.25,na.rm=TRUE),digits),nsmall=digits),"-",format(round(quantile(var,0.75,na.rm=TRUE),digits),nsmall=digits),")",sep="")
}
meanSD <- function(var,digits=1){
  paste(format(round(mean(var,na.rm=TRUE),digits),nsmall=digits)," (",format(round(sd(var,na.rm=TRUE),digits),nsmall=digits),")",sep="")
}


## FORCE NUMBER will take a factor/character to a number without issuing the warnings
forcenumber <- function(var){
    options(warn = -1)
    x <- as.numeric(gsub("[^0-9.]","",as.character(var)))
    options(warn = 1)
    return(x)
}

## PRINT DIAGNOSTIC ACCURACY
calcprop <- function(x,n){
 p <- binconf(x,n,alpha=0.05,method="wilson")
 if(length(p)==3)  p1 <- paste(myformat2(p[1])," (",myformat2(p[2]),", ",myformat2(p[3]),") [",x,"/",n,"]",sep="")
 if(length(p)>3)  p1 <- paste(myformat2(p[,1])," (",myformat2(p[,2]),", ",myformat2(p[,3]),") [",x,"/",n,"]",sep="")
 return(p1)
}
getacctex <- function(TP,TN,FP,FN,by,label,rowlabel,indent=TRUE){
  if(length(rowlabel) != sum(!(unique(by)%in% NA))){warning("Function GETACCTEX: number of unique values does not match length of rowlabel")}
  SN1 <- table(TP,by)[2,]
  SN2 <- table(TP | FN,by)[2,]
  SN <- calcprop(SN1,SN2)
  SP1 <- table(TN,by)[2,]
  SP2 <- table(FP | TN,by)[2,]
  SP <- calcprop(SP1,SP2)
  PPV1 <- table(TP,by)[2,]
  PPV2 <- table(TP | FP,by)[2,]
  PPV <- calcprop(PPV1,PPV2)
  NPV1 <- table(TN,by)[2,]
  NPV2 <- table(FN | TN,by)[2,]
  NPV <- calcprop(NPV1,NPV2)
  getn <- table(TP|TN|FP|FN,by)
  lines1 <- ifelse(indent,paste(paste(paste("\\\\hspace{.2in} ",rowlabel," (n=",getn,")",sep=""),SN,SP,PPV,NPV,sep=" & "),collapse=" \\\\\\\\ "),paste(paste(paste(rowlabel," (n=",getn,")",sep=""),SN,SP,PPV,NPV,sep=" & "),collapse=" \\\\\\\\ "))
  lines <- gsub("NaN","--",ifelse(!is.na(label),paste(c(label,lines1),collapse=" \\\\\\\\ "),lines1))
  return(lines)
}

## PRINT LATEX HEADER FOR GETNPTEX, GETMEDTEX, and GETNP1LINE functions
printheader <- function(crosstab,total=TRUE,total.label="Combined",pval=FALSE,pval.label="P-value"){
if(total){
  x <- paste(c("",names(crosstab),total.label),collapse=" & ")
  y <- paste(c("",paste("(n=",c(crosstab,sum(crosstab)),")",sep="")),collapse=" & ")
  }
if(!total){
  x <- paste(c("",names(crosstab)),collapse=" & ")
  y <- paste(c("",paste("(n=",c(crosstab),")",sep="")),collapse=" & ")
  }
if(pval){x <- paste(c(x,pval.label),collapse=" & ")}
  line <- paste(c(x,y),collapse=" \\\\\\\\ ")
  return(line)
}


## PRINT ONE LINE FOR VARIABLES WITH TWO CATEGORIES OR ONLY ONE OF INTEREST 
## EXAMPLE (PRINT FEMALE (%) -- NO NEED TO PRINT MALE(%) AS WELL)
# var must be a variable that takes on 1 value and the rest are NA
# similarly, rowlabel must be length 1
getnp1line <- function(var,by,rowlabel,indent=TRUE,inc.pval=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE){
  myper1 <- get(formatper)
  n <- table(var,by)
  d <- table(by)
  n1 <- table(var)
  if(!rowper){p <- sapply(100*n[1,]/d,myper1)}
  if(rowper){p <- sapply(100*n[1,]/n1,myper1)}
  pt <- sapply(100*n1/length(by),myper1)
  if(indent==TRUE){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  else{line <- paste(paste(rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  if(inc.pval==TRUE){
    warning("Function NP1LINE: assuming if var!=1 then var=0 for hypothesis test")
    var1 <- ifelse(is.na(var),0,var)
    if(by.order==TRUE){pval <- myformat4(wilcox.test(as.numeric(by)~ var1)$p.value)}
    if(by.order==FALSE){pval <- myformat4(chisq.test(table(var1,by))$p.value)}
    line <- paste(line,pval,sep=" & ")
    }
  return(list(line=line))
}


## PRINT MED (IQR) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order'' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getmedtex <- function(var,by,label,note=note,inc.pval=TRUE,lower.label=TRUE,by.order=FALSE,digits=8,formatper="myper0"){
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var,digits=digits),1)
  text <- paste(s[3]," (",s[2],", ",s[5],")",sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]],digits=digits)[1:5],1)
      textby[i] <- ifelse(!is.na(sby[i,3]),paste(sby[i,3]," (",sby[i,2],", ",sby[i,5],")",sep=""), "--")
  }
  if(inc.pval==TRUE){
      if(length(level)==1) {pval <- " "}
      else if(length(level)>1 & by.order==FALSE) {pval <- myformat4(kruskal.test(var ~ by)$p.value)}
      else if(length(level)>1 & by.order==TRUE)  {pval <- myformat4(cor.test(var, as.numeric(by),alternative="two.sided",method="spearman")$p.value)}
      
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,pval,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,pval,sep=' & ')
      }
  } else{
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,sep=' & ')
      }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing ",tolower(label),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
      else{line2 <- paste(paste("\\\\hspace{.1in}Missing ",label,", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}
           
           

## PRINT N (%) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order' and 'var.order' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getnptex <- function(var,by,label,rowlabel=rowlabel,note=note,inc.pval=TRUE,var.order=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE){
  myper1 <- get(formatper)
  if(missing(rowlabel)){
    if(is.factor(var)){rowlabel <- levels(var)}
    if(is.character(var)){rowlabel <- names(table(var))}
  }
  if(length(rowlabel) != sum(!(unique(var)%in% NA))){warning("Function GETNPTEX: number of unique values does not match length of rowlabel")}
  n <- table(var,by)
  n1 <- rowSums(n)
  d <- colSums(table(var,by)[1:nrow(n),])
  p <- matrix(nrow=nrow(n), ncol=ncol(n))
  pt <- rep(NA,nrow(n))
  lines <- rep(NA,nrow(n))
  for(i in 1:nrow(n)){
    p[i,which(d==0)] <- 0
    if(!rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/d[which(d!=0)],myper1)}
    if(rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/n1[i],myper1)}
    pt[i] <- sapply(100*table(var)[i]/sum(table(var)[1:nrow(n)]),myper1)
    lines[i] <- paste(paste("\\\\hspace{.2in}",rowlabel[i]),paste(paste(n[i,]," (",p[i,],"\\\\%)",sep=""),collapse=" & "),paste(sum(n[i,])," (",pt[i],"\\\\%)",sep=""),sep=" & ")
  }
# ADD PVALUE TO COLUMN HEADING IF DESIRED
  if(inc.pval==TRUE){
    if(by.order==TRUE){pval <- myformat4(kruskal.test(as.numeric(by), var)$p.value)}
    if(var.order==TRUE){pval <- myformat4(kruskal.test(var, by)$p.value)}
    if(by.order==TRUE & var.order==TRUE){pval <- myformat4(cor.test(var, by, alternative="two.sided", method="spearman")$p.value)} # ADDED 12/30/2010
    if(by.order==FALSE & var.order==FALSE){pval <- myformat4(chisq.test(n[n1!=0,d!=0])$p.value)}
    spaces <- rep(" ",1+length(table(by)))
    line1 <- paste(paste(label,", n(\\\\%)",sep=""),paste(spaces,collapse=" & "),pval,sep=" & ")
  } else{
    line1 <- paste(label,", n(\\\\%)",sep="")
  }
  all.lines <- paste(line1,paste(lines[which(n1!=0)],collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(missing(note)){ line2 <- paste("\\\\hspace{.1in}Missing",paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    } else{line2 <- paste(paste("\\\\hspace{.1in}Missing\\\\tnote{",note,"}",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    }
    all.lines.miss <- paste(line1,line2,paste(lines,collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
    return(list(line=all.lines,line.incmiss=all.lines.miss))
  }
  else{return(list(line=all.lines))}
}

simpleCap <- function(s) {
         paste(toupper(substring(s, 1,1)), tolower(substring(s, 2)),
               sep="", collapse=" ")
     }


# find N, percent by some variable and overall
getnp <- function(var,by){
  n <- table(var,by)
  d <- colSums(table(var,by)[1:nrow(n),])
  p <- matrix(nrow=nrow(n), ncol=ncol(n))
  pt <- rep(NA,nrow(n))
  pval <- myformat3(chisq.test(n)$p.value)

  for(i in 1:nrow(n)){
    p[i,] <- sapply(100*n[i,]/d,myper1)
    pt[i] <- sapply(100*table(var)[i]/sum(table(var)[1:nrow(n)]),myper1)
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop<- round(no/table(by),3)*100   
    nopt <- round(sum(no)/sum(table(by)),3)*100   
    return(list(n=n,p=p,pt=pt,pval=pval,no=no,nop=nop,nopt=nopt))
  }
  else{return(list(n=n,p=p,pt=pt,pval=pval))}
}

# find median (Q1,Q3) missing N, percent by some variable and overall
getmed <- function(var,by){
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),2)
  text <- paste(s[3]," (",s[2],", ",s[5],")",sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s))
  textby <- rep(NA,length(level))
  if(length(level)==1) {}
  else if(length(level)==2){pval <- myformat3(wilcox.test(var ~ by)$p.value)}
  else if(length(level)>2) {pval <- myformat3(kruskal.test(var ~ by)$p.value)}
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],2)
      textby[i] <- paste(sby[i,3]," (",sby[i,2],", ",sby[i,5],")",sep="")    
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop<- round(no/table(by),3)*100   
    nopt <- round(sum(no)/sum(table(by)),3)*100   
    return(list(text=text,textby=textby,pval=pval,no=no,nop=nop,nopt=nopt))
  }
  else{return(list(text=text,textby=textby,pval=pval))}
}


# LONGITUDINAL VERSION OF GETMEDTEX (developed for NEMART)
# when by is a timepoint like visit #, then we do not want a total column
# additionally we need to change the test of significance
# MUST RUN library(nlme)
longmedtex <- function(var,by,caseid,label,note=note,inc.pval=TRUE,lower.label=TRUE,formatper="myper0"){
  myper1 <- get(formatper)
  s <- round(summary(var),1)
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  if(length(level)==1) {pval <- " "}
#  else if(length(level)>1) {
#      model1 <- lmer(var ~ by + (by|caseid))
#      model2 <- lmer(var ~ by + (1|caseid) + (0+by|caseid))
#      pval <- myformat3(anova(model1,model2)$'Pr(>Chisq)'[2])
#  }
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],1)
      if(!is.na(sby[i,3])){textby[i] <- paste(sby[i,3]," (",sby[i,2],", ",sby[i,5],")",sep="")}
      else if(is.na(sby[i,3])){textby[i] <- "-"}
  }
  if(inc.pval==TRUE & length(level)>1){
      newdata1 <- data.frame(var,by,caseid)[!is.na(var),]
      newdata <- groupedData(var ~ by | caseid, data=newdata1)
      mylme <- lme(var ~ by, data=newdata, random=~1|caseid, na.action=na.omit,method="ML")
      pval <- myformat4(as.data.frame(anova(mylme))$'p-value'[2])
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),pval,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),pval,sep=' & ')
      }
  } else{
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),sep=' & ')
      }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing ",tolower(label),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),sep=" & ")}
      else{line2 <- paste(paste("\\\\hspace{.1in}Missing ",label,", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}


######################
## ADDED 2011-06-20 ##
######################
## GETNPUNI AND GETMEDUNI basically produce the 'combined' only column in case I do not have a variable of interest for crosstabulation - just drop 'by'
getmeduni <- function(var,label,note=note,lower.label=TRUE,digits=8,formatper="myper0"){
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var,digits=digits),1)
  text <- paste(s[3]," (",s[2],", ",s[5],")",sep="")

  if(missing(note)){line1 <- paste(label,text,sep=' & ')
      } 
  else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),text,sep=' & ')
      }
  if(sum(is.na(var))>0){
    no <- sum(is.na(var))
    nop <- sapply(100*no/length(var),myper1)
    if(lower.label==TRUE){
	line2 <- paste(paste("\\\\hspace{.1in}Missing ",tolower(label),", n(\\\\%)",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    else{
	line2 <- paste(paste("\\\\hspace{.1in}Missing ",label,", n(\\\\%)",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}

getnpuni <- function(var,label,rowlabel=rowlabel,note=note,formatper="myper0"){
  myper1 <- get(formatper)
  if(missing(rowlabel)){
    if(is.factor(var)){rowlabel <- levels(var)}
    if(is.character(var)){rowlabel <- names(table(var))}
  }
  if(length(rowlabel) != sum(!(unique(var)%in% NA))){warning("Function GETNPUNI: number of unique values does not match length of rowlabel")}
  n <- table(var)
  p <- sapply(100*n/sum(n),myper1)

  lines <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  line1 <- paste(label,", n(\\\\%)",sep="")

  all.lines <- paste(line1,paste(lines[which(n!=0)],collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
  if(sum(is.na(var))>0){
    no <- sum(is.na(var))
    nop <- sapply(100*no/length(var),myper1)
    if(missing(note)){ 
      line2 <- paste("\\\\hspace{.1in}Missing",paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    } else{
      line2 <- paste(paste("\\\\hspace{.1in}Missing\\\\tnote{",note,"}",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    all.lines.miss <- paste(line1,line2,paste(lines,collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
    return(list(line=all.lines,line.incmiss=all.lines.miss))
  }
  else{return(list(line=all.lines))}
}

getnp1lineuni <- function(var,rowlabel,indent=TRUE,formatper="myper0"){
  myper1 <- get(formatper)
  n <- table(var)
  p <- sapply(100*n/length(var),myper1)
  if(indent==TRUE){
      line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  }
  else{
      line <- paste(paste(rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  }
  return(list(line=line))
}
